/*
 * FLFT v1.0 by Nuck-TH
 *
 * Based on FLST (Firefox 4+) v1.2.2
 * by Fabian Schneider,
 * which in turn is based on older
 * add-ons by "Gorgias" and "Gozer"
*/


var flft = {
      enabledPref: 'extensions.flft.enabled',
      flstPref: 'extensions.flft.flst',
      tabflipPref: 'extensions.flft.tabflipping',
      fastflipPref: 'extensions.flft.fastflip',
      middleclickPref: 'extensions.flft.middleclickrestore',
      enablekeyPref: 'extensions.flft.enablekey',

      suppressMouseUp: new Boolean(),

      observe: function(subject, topic, data)
      {
        if (topic != 'nsPref:changed')
          return;

        switch (data) {
          case this.enabledPref:
            this.enabled = this.getPref('Bool', this.enabledPref, true);
            break;

          case this.flstPref:
            this.flst = this.getPref('Bool', this.flstPref, true);
            break;

          case this.tabflipPref:
            this.tabflipping = this.getPref('Bool', this.tabflipPref, true);
            break;

          case this.fastflipPref:
            this.fastflipping = this.getPref('Bool', this.fastflipPref, false);

            if (this.fastflipping)
             {
             gBrowser.tabContainer.removeEventListener('click', flft.tabClick, false);
             gBrowser.tabContainer.removeEventListener('mousedown', flft.dragDropHelper, false);


             gBrowser.tabContainer.addEventListener('mousedown', flft.tabClick, false);
             }
            else
             {
             gBrowser.tabContainer.removeEventListener('mousedown', flft.tabClick, false);

             gBrowser.tabContainer.addEventListener('click', flft.tabClick, false);
             gBrowser.tabContainer.addEventListener('mousedown', flft.dragDropHelper, false);
             }

            break;

          case this.middleclickPref:
            this.middleclickrestore = this.getPref('Bool', this.middleclickPref, true);
            break;

          case this.enablekeyPref:
            this.enablekey = this.getPref('Bool', this.enablekeyPref, true);
            break;

          default:
            break;
        }
      },

      getPref: function(type, pref, def)
      {
        try { return this.prefs['get' + type + 'Pref'](pref); } catch (e) { }

        return def;
      },
      setPref: function(type, pref, val)
      {
        try {
          this.prefs['set' + type + 'Pref'](pref, val);
        } catch (e) {
          alert(e);
        }
      },

      tabClose: function(e)
      {
        if (!this.enabled)
          return;

        if (!this.flst)
          return;

        var tab = e.target;
        var tabs = tab.parentNode.childNodes;

        var index = 0;
        var last  = 0;

        for (var i = 0; i < tabs.length; i++) {
          var s = tabs[i].getAttribute('lastselected');

          if (s && s > last && tabs[i] != tab) {
            index = i;
            last = s;
          }
        }

        gBrowser.selectedTab = tabs[index];
      },


      tabMiddleClick: function(e)
      {
        if (!this.enabled)
          return;

        if (!this.middleclickrestore)
          return;

        if ((e.button == 1) &&
           (e.originalTarget.localName == 'box')) {
             undoCloseTab();  // middle click on tab bar restores recently closed tab.
             e.preventDefault();
             e.stopPropagation();
        }

      return;
      },


      tabClick: function(e)
      {

        if (!flft.enabled)
          return;

        if (!flft.tabflipping)
          return;

        if (!e.button == 0) // Only left-click will be processed.
          return;

        if ((e.originalTarget.className == 'tab-close-button') || (e.originalTarget.className == 'tab-close-button close-icon') || (e.originalTarget.className == 'tab-icon-sound'))
          return;

        if (flft.suppressMouseUp) {
          flft.suppressMouseUp = false;
          return;
        }

        var source = e.target;
        var tabs = gBrowser.tabContainer.childNodes;
        var activeTab = gBrowser.selectedTab;
        var className = e.originalTarget.className;
        var localName = e.originalTarget.localName;
        var justSelected = (Date.now() - activeTab.getAttribute('lastselected'));

        if ((source == activeTab) && (justSelected >= 270) &&
          (!((className == 'tab-close-button') || (className == 'tab-close-button close-icon')) ||
          (localName == 'tab'))) {

           flft.tabFlip(false);

           e.preventDefault();
           e.stopPropagation();

        } else {
          if (gBrowser.selectedTab != e.target)
            gBrowser.selectedTab = e.target;
        }

      },


      dragDropHelper: function(e)
      {

        if (!flft.enabled)
          return;

        if (!flft.tabflipping)
          return;

        if (!e.button == 0) // Only left-click will be processed.
          return;

        var source = e.target;
        var activeTab = gBrowser.selectedTab;
        var className = e.originalTarget.className;
        var localName = e.originalTarget.localName;
        var justSelected = (Date.now() - activeTab.getAttribute('lastselected'));

        if (((source != activeTab) &&
          (!((className == 'tab-close-button') || (className == 'tab-close-button close-icon')) ||
          (localName == 'tab'))) || ((source == activeTab) && (justSelected < 270))) {

          flft.suppressMouseUp = true;

          }
        return;
      },


      tabFlip: function(byKbd)
      {

        if (!flft.enabled)
          return;

        if (byKbd && (!flft.enablekey))
          return;

        if (!flft.tabflipping)
          return;

        var tab = gBrowser.selectedTab;
        var tabs = tab.parentNode.childNodes;
    
        var index = 0;
        var last  = 0;

        for (var i = 0; i < tabs.length; i++) {
          var s = tabs[i].getAttribute('lastselected');

          if (s && s > last && tabs[i] != tab) {
            index = i;
            last = s;
          }
        }

        gBrowser.selectedTab = tabs[index];

      },


      tabSelect: function(e)
      {
        var tab = e.target;
     
        tab.setAttribute('lastselected', Date.now());
      },


      startup: function()
      {
        this.prefs = Components.classes['@mozilla.org/preferences-service;1']
                       .getService(Components.interfaces.nsIPrefBranch2);

        this.prefs.addObserver(this.enabledPref, this, false);
        this.prefs.addObserver(this.flstPref, this, false);
        this.prefs.addObserver(this.tabflipPref, this, false);
        this.prefs.addObserver(this.fastflipPref, this, false);
        this.prefs.addObserver(this.middleclickPref, this, false);
        this.prefs.addObserver(this.enablekeyPref, this, false);

        this.enabled = this.getPref('Bool', this.enabledPref, true);
        this.flst = this.getPref('Bool', this.flstPref, true);
        this.tabflipping = this.getPref('Bool', this.tabflipPref, true);
        this.fastflipping = this.getPref('Bool', this.fastflipPref, false);
        this.middleclickrestore = this.getPref('Bool', this.middleclickPref, true);
        this.enablekey = this.getPref('Bool', this.enablekeyPref, true);

        gBrowser.tabContainer.addEventListener('TabClose',
          function(e) {
            flft.tabClose(e);
          }, false);

        gBrowser.tabContainer.addEventListener('TabSelect',
          function(e) {
            flft.tabSelect(e);
          }, false);

        if (this.fastflipping)
         {
         gBrowser.tabContainer.addEventListener('mousedown', flft.tabClick, false);
         }
        else
         {
         gBrowser.tabContainer.addEventListener('click', flft.tabClick, false);
         gBrowser.tabContainer.addEventListener('mousedown', flft.dragDropHelper, false);
         }

        gBrowser.mTabContainer.addEventListener('click',
          function(e) {
            flft.tabMiddleClick(e);
          }, true);

        options_startup();

      },
      shutdown: function()
      {
        this.prefs.removeObserver(this.enabledPref, this);
        this.prefs.removeObserver(this.flstPref, this);
        this.prefs.removeObserver(this.tabflipPref, this);
        this.prefs.removeObserver(this.fastflipPref, this);
        this.prefs.removeObserver(this.middleclickPref, this);
        this.prefs.removeObserver(this.enablekeyPref, this);

        options_shutdown();
      }
    };

    Components.utils.import("chrome://flft/content/options.jsm");

    window.addEventListener('load', function(event) {
        flft.startup();
      }, false);

    window.addEventListener('unload', function(event) {
        flft.shutdown();
      }, false);

